package gonganxitong

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
)

type apply struct {
	Table
	user      user
	userPlace userPlace
	place     place
	street    street
	event     event
	portal    string `data:"true"`
	create    string `name:"create place_uid*:select user_place_role*:select begin_time:select@date end_time:select@date" role:"void"`
	cancel    string `name:"cancel" role:"void"`
	submit    string `name:"submit" role:"void"`
	list      string `name:"list place_uid uid auto" role:"void"`
}

func (s apply) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.PLACE_UID:
		m.Cmdy(s.userPlace, m.Option(model.USER_UID))
		m.Cut(model.PLACE_UID, model.PLACE_NAME).DisplayInputKeyNameIconTitle()
	case model.USER_PLACE_ROLE:
		for k, v := range UserPlaceRoleList {
			if k != UserPlaceCreator && k != UserPlaceVisitor {
				m.Push(arg[0], k).Push(model.NAME, v)
			}
		}
		m.SortInt(arg[0]).DisplayInputKeyNameIconTitle()
	default:
		s.Table.Inputs(m, arg...)
	}
}
func (s apply) Create(m *ice.Message, arg ...string) {
	arg = kit.TransArgKeys(arg, model.USER_PLACE_ROLE, model.ROLE)
	arg = kit.TransArgValueTime(arg, model.BEGIN_TIME, model.END_TIME)
	s.Table.Create(m, kit.Simple(arg, m.OptionSimple(model.USER_UID), model.STATUS, ApplyCreate)...)
}
func (s apply) Cancel(m *ice.Message, arg ...string) {
	msg := s.status(m, ApplyCreate, ApplyCancel)
	s.SendTemplate(m, msg.Append(model.USER_UID), "场所权限申请 已取消")
}
func (s apply) Submit(m *ice.Message, arg ...string) {
	defer s.event.Record(m, m.Option(model.UID))
	defer m.ToastProcess()()
	role := UserPlaceCreator
	switch UserPlaceRole(kit.Int(s.status(m, ApplyCreate, ApplySubmit).Append(model.ROLE))) {
	case UserPlaceLandlord:
		role = UserPlaceCreator
	case UserPlaceTenant, UserPlaceAdmin:
		role = UserPlaceLandlord
	}
	msg := m.Cmd(s.userPlace, s.userPlace.Select, m.OptionSimple(model.PLACE_UID), model.ROLE, role.String())
	m.Cmd(order{}, order{}.Create, msg.AppendSimple(model.USER_UID), model.APPLY_UID, m.Option(model.UID), model.STATUS, OrderCreate)
	s.SendTemplate(m, m.Option(model.USER_UID), "场所权限申请 已提交 🕑")
}
func (s apply) List(m *ice.Message, arg ...string) {
	s.Tables(m, s.place, s.street).Fields(m, "applies.created_at",
		model.PLACE_NAME, model.PLACE_TYPE, "applies.role AS user_place_role", model.APPLY_STATUS,
		model.STREET_NAME, model.PLACE_ADDRESS, "applies.uid", model.PLACE_UID, model.STREET_UID,
		model.BEGIN_TIME, model.END_TIME,
	).Orders(m, "created_at DESC")
	if len(arg) == 0 {
		if m.IsTech() {
			s.Table.Select(m)
		} else {
			s.Table.Select(m, m.OptionSimple(model.USER_UID)...)
		}
	} else if len(arg) == 1 {
		if s.Table.Select(m, model.USER_UID, m.Option(model.USER_UID), model.PLACE_UID, arg[0]).Action(s.Create); m.Length() == 0 {
			m.EchoInfoButton(m.Trans("please create apply", "请创建权限申请"), s.Create)
			return
		}
		defer m.Sort("apply_status,created_at", []string{
			ApplyCreate.String(),
			ApplySubmit.String(),
			ApplyApproved.String(),
			ApplyRejected.String(),
			ApplyCancel.String(),
		}, ice.STR_R)
	} else {
		s.Table.Select(m.FieldsSetDetail(), "applies.uid", arg[1])
		if ApplyStatus(kit.Int(m.Append(model.APPLY_STATUS))) == ApplySubmit {
			m.EchoQRCode(cmdurl(m, arg[0], m.Prefix("order"), arg[1]))
		}
	}
	m.Table(func(value ice.Maps) {
		switch ApplyStatus(kit.Int(value[model.APPLY_STATUS])) {
		case ApplyCreate:
			m.PushButton(s.Cancel, s.Submit)
		default:
			m.PushButton()
		}
	})
	s.RewriteAppend(m).Display("").DisplayCSS("")
}

func init() { ice.Cmd(prefixKey(), apply{}) }

func (s apply) Reject(m *ice.Message, arg ...string) {
	msg := s.status(m, ApplySubmit, ApplyRejected)
	s.SendTemplate(m, msg.Append(model.USER_UID), "场所权限申请 已驳回 ❌")
}
func (s apply) Approve(m *ice.Message, arg ...string) {
	msg := s.status(m, ApplySubmit, ApplyApproved)
	s.SendTemplate(m, msg.Append(model.USER_UID), "场所权限申请 已通过 ✅")
	m.Cmd(s.userPlace, s.userPlace.Create, msg.AppendSimple(model.USER_UID, model.PLACE_UID, model.ROLE))
}
func (s apply) status(m *ice.Message, from, todo ApplyStatus, arg ...string) *ice.Message {
	msg := s.Table.Select(m.Spawn(), m.OptionSimple(model.UID)...)
	if !m.WarnNotFound(msg.Length() == 0, m.Option(model.UID)) {
		if status := ApplyStatus(kit.Int(msg.Append(model.STATUS))); !m.WarnNotValid(status != from, status.String()) {
			s.Table.Update(m, kit.Dict(model.STATUS, todo), m.OptionSimple(model.UID)...)
		}
	}
	return msg
}

type ApplyStatus int

const (
	ApplyCreate ApplyStatus = iota
	ApplyCancel
	ApplySubmit
	ApplyRejected
	ApplyApproved
)

var ApplyStatusList = map[ApplyStatus]string{
	ApplyCreate:   "create",
	ApplyCancel:   "cancel",
	ApplySubmit:   "submit",
	ApplyRejected: "rejected",
	ApplyApproved: "approved",
}

func (s ApplyStatus) String() string { return ApplyStatusList[s] }
