package model

import (
	"time"

	"shylinux.com/x/ice"
	"shylinux.com/x/mysql-story/src/db"
)

const (
	UID             = "uid"
	NAME            = "name"
	TYPE            = "type"
	ROLE            = "role"
	STATUS          = "status"
	ADDRESS         = "address"
	EMAIL           = "email"
	AVATAR          = "avatar"
	OPEN_ID         = "open_id"
	USER_UID        = "user_uid"
	USER_NAME       = "user_name"
	USER_PLACE_ROLE = "user_place_role"
	PLACE_UID       = "place_uid"
	PLACE_NAME      = "place_name"
	PLACE_TYPE      = "place_type"
	PLACE_ADDRESS   = "place_address"
	STREET_UID      = "street_uid"
	STREET_NAME     = "street_name"
	APPLY_UID       = "apply_uid"
	APPLY_STATUS    = "apply_status"
	ORDER_STATUS    = "order_status"
	ORDER_UID       = "order_uid"
	BEGIN_TIME      = "begin_time"
	END_TIME        = "end_time"
)

type Sess struct {
	db.ModelWithUID
	UserUID string `gorm:"type:char(32)"`
	IP      string `gorm:"type:char(16)"`
	Agent   string `gorm:"size:char(256)"`
}
type User struct {
	db.ModelWithUID
	OpenID string `gorm:"type:char(32)";index`
	Email  string `gorm:"type:char(32)"`
	Name   string `gorm:"type:char(32)"`
	Avatar string `gorm:"size:256"`
}
type UserPlace struct {
	db.Model
	UserUID   string `gorm:"type:char(32);index"`
	PlaceUID  string `gorm:"type:char(32)"`
	Role      uint8
	BeginTime time.Time
	EndTime   time.Time
}
type Place struct {
	db.ModelWithUID
	StreetUID string `gorm:"type:char(32)"`
	Name      string `gorm:"type:char(32)"`
	Type      uint8
	Address   string `gorm:"size:256"`
}
type Street struct {
	db.ModelWithUID
	Name string `gorm:"type:char(32)"`
}

type Event struct {
	db.ModelWithUID
	PlaceUID string `gorm:"type:char(32);index"`
	UserUID  string `gorm:"type:char(32)"`
	Index    string `gorm:"type:char(32)"`
	Operate  string `gorm:"type:char(32)"`
	Args     string `gorm:"type:char(32)"`
}
type Apply struct {
	db.ModelWithUID
	UserUID   string `gorm:"type:char(32);index"`
	PlaceUID  string `gorm:"type:char(32)"`
	Status    uint8
	Role      uint8
	BeginTime time.Time
	EndTime   time.Time
}
type Order struct {
	db.ModelWithUID
	UserUID  string `gorm:"type:char(32);index"`
	ApplyUID string `gorm:"type:char(32);index"`
	Status   uint8
}

type models struct{ db.Models }

func (s models) Init(m *ice.Message, arg ...string) {
	s.Models.Register(m, "gonganxitong", &Sess{}, &User{}, &UserPlace{}, &Place{}, &Street{}, Event{}, Apply{}, Order{})
}

func init() { ice.Cmd("web.team.community.gonganxitong.models", models{}) }
