package gonganxitong

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
)

type order struct {
	Table
	user    user
	apply   apply
	place   place
	street  street
	event   event
	portal  string `data:"true"`
	create  string `name:"create user_uid* apply_uid* status*"`
	reject  string `name:"reject" role:"void"`
	approve string `name:"approve" role:"void"`
	list    string `name:"list place_uid uid auto" role:"void"`
}

func (s order) Create(m *ice.Message, arg ...string) {
	s.Table.Create(m, arg...)
	s.SendTemplate(m, m.Option(model.USER_UID), "场所权限申请 请审批 🕑")
}
func (s order) Reject(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	defer s.event.Record(m, m.Option(model.UID))
	m.Cmd(s.apply, s.apply.Reject, m.Option(model.UID))
	s.Table.Update(m, kit.Dict(model.STATUS, OrderRejected), model.UID, m.Option(model.ORDER_UID))
	s.SendTemplate(m, m.Option(model.USER_UID), "场所权限申请 已驳回 ❌")
}
func (s order) Approve(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	defer s.event.Record(m, m.Option(model.UID))
	m.Cmd(s.apply, s.apply.Approve, m.Option(model.UID))
	s.Table.Update(m, kit.Dict(model.STATUS, OrderApproved), model.UID, m.Option(model.ORDER_UID))
	s.SendTemplate(m, m.Option(model.USER_UID), "场所权限申请 已通过 ✅")
}
func (s order) List(m *ice.Message, arg ...string) {
	s.Tables(m, s.apply, s.place, s.street).Fields(m, "orders.created_at",
		model.PLACE_NAME, model.PLACE_TYPE, "applies.role AS user_place_role", model.ORDER_STATUS,
		model.STREET_NAME, model.PLACE_ADDRESS, model.PLACE_UID, "orders.uid AS order_uid",
		"applies.uid", model.BEGIN_TIME, model.END_TIME,
	).Orders(m, "created_at DESC")
	if len(arg) == 0 {
		if m.IsTech() {
			s.Table.Select(m)
		} else {
			s.Table.Select(m, "orders.user_uid", m.Option(model.USER_UID))
		}
	} else if len(arg) == 1 {
		s.Table.Select(m, "orders.user_uid", m.Option(model.USER_UID), model.PLACE_UID, arg[0]).Action(s.Create)
		defer m.Sort("order_status,created_at", []string{OrderCreate.String(), OrderApproved.String(), OrderRejected.String()}, ice.STR_R)
	} else {
		s.Table.Select(m.FieldsSetDetail(), "orders.user_uid", m.Option(model.USER_UID), "applies.uid", arg[1])
	}
	m.Table(func(value ice.Maps) {
		switch OrderStatus(kit.Int(value[model.ORDER_STATUS])) {
		case OrderCreate:
			m.PushButton(s.Reject, s.Approve)
		default:
			m.PushButton()
		}
	})
	s.RewriteAppend(m).Display("").DisplayCSS("")

}
func init() { ice.Cmd(prefixKey(), order{}) }

type OrderStatus int

const (
	OrderCreate OrderStatus = iota
	OrderRejected
	OrderApproved
)

var OrderStatusList = map[OrderStatus]string{
	OrderCreate:   "create",
	OrderRejected: "rejected",
	OrderApproved: "approved",
}

func (s OrderStatus) String() string { return OrderStatusList[s] }
