package gonganxitong

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/ctx"
	"shylinux.com/x/icebergs/base/mdb"
	"shylinux.com/x/icebergs/base/web"
	"shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
)

type portal struct {
	ice.Hash
	user        user
	userPlace   userPlace
	place       place
	street      street
	export      string `data:"true"`
	short       string `data:"index"`
	field       string `data:"time,name,icons,index,order,enable,type,role"`
	list        string `name:"list place_uid index uid auto" role:"void"`
	placeCreate string `name:"placeCreate street_uid*:select place_type*:select name* address*" role:"void"`
	placeRemove string `name:"placeRemove place_uid*" role:"void"`
	scanQRCode  string `name:"scanQRCode type text" role:"void"`
}

func (s portal) Inputs(m *ice.Message, arg ...string) {
	s.place.Inputs(m, arg...)
}
func (s portal) List(m *ice.Message, arg ...string) {
	if m.Option("form") == "table" {
		s.Hash.List(m, arg...).SortInt(mdb.ORDER)
		return
	}
	if len(arg) == 0 {
		m.Cmdy(s.userPlace, m.Option(model.USER_UID)).Action(s.PlaceCreate, s.ScanQRCode).PushAction(s.PlaceRemove)
		kit.If(!m.IsErr() && m.Length() == 0, func() {
			m.EchoInfoButton(m.Trans("Please Create Your Place", "请添加场所"), s.PlaceCreate, s.ScanQRCode)
		})
	} else if len(arg) == 1 {
		s.Hash.List(m, arg[1:]...).SortInt(mdb.ORDER)
	} else if len(arg) == 2 {
		m.Cmdy(ctx.COMMAND, arg[1]).Push(ctx.ARGS, arg[0])
	} else {
		m.FieldsSetDetail().Cmdy(arg[1], mdb.SELECT, model.UID, arg[2]).PushAction().Action()
	}
	m.Display("").DisplayCSS("")
}
func (s portal) PlaceCreate(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	if m.WarnNotFound(m.Cmd(s.street, s.street.Select, model.UID, m.Option(model.STREET_UID)).Length() == 0, "street") {
		return
	}
	arg = kit.TransArgKeys(arg, model.PLACE_TYPE, model.TYPE)
	if m.Cmdy(s.place, s.place.Create, arg).IsErr() {
		return
	}
	args := kit.Simple(model.USER_UID, m.Option(model.USER_UID), model.PLACE_UID, m.Result(), model.ROLE, UserPlaceCreator)
	m.Cmdy(s.userPlace, s.userPlace.Create, args)
	m.ProcessRefresh()
}
func (s portal) PlaceRemove(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	args := kit.Simple(model.USER_UID, m.Option(model.USER_UID), model.PLACE_UID, m.Option(model.PLACE_UID))
	msg := m.Cmd(s.userPlace, s.userPlace.Select, args)
	if m.WarnNotFound(msg.Length() == 0, "place") {
		return
	}
	m.Cmdy(s.userPlace, s.userPlace.Delete, args)
	if kit.Int(msg.Append(model.ROLE)) == kit.Int(UserPlaceCreator) {
		m.Cmdy(s.place, s.place.Delete, model.UID, m.Option(model.PLACE_UID))
	}
	m.ProcessRefresh()
}
func (s portal) ScanQRCode(m *ice.Message, arg ...string) {
	defer m.ToastProcess()()
	if m.Option(mdb.TYPE) == mdb.TEXT {
		args := kit.Simple(m.OptionSimple(model.USER_UID), model.PLACE_UID, m.Option(mdb.TEXT))
		m.Cmdy(s.userPlace, s.userPlace.Create, args, model.ROLE, UserPlaceVisitor)
	}
	if m.Option(mdb.TYPE) == web.LINK {
		args := m.ParseURL(m.Option(mdb.TEXT))
		if len(args) > 1 && args[1] == m.Prefix("apply") {
			args := kit.Simple(m.OptionSimple(model.USER_UID), model.PLACE_UID, args[0])
			m.Cmdy(s.userPlace, s.userPlace.Create, args, model.ROLE, UserPlaceVisitor)
		}
	}
}

func init() { ice.Cmd(prefixKey(), portal{}) }

type Portal struct {
	portal
	export string `data:"true"`
	short  string `data:"index"`
	field  string `data:"time,name,icons,index,order,enable,type,role"`
}

func init() { ice.Cmd(prefixKey(), Portal{}) }

func (s portal) Show(m *ice.Message, arg ...string) {
	m.GoSleep("30ms", func() {
		cmd := m.GetCommand()
		icon := kit.Select(cmd.Icon, kit.Value(cmd.Meta, kit.Keys(ice.CTX_ICONS, m.CommandKey())))
		m.Cmd(s, s.Create, mdb.NAME, cmd.Help, mdb.ICONS, icon, ctx.INDEX, m.PrefixKey())
	})
}
